/*
 * Copyright (c) 1996 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * route.h
 */


#ifndef ROUTE_H
#define ROUTE_H
#ifdef IPSEC
#include "keylist.h"
#define ROSA_AH		0
#define ROSA_ESP	1
#define ROSA_MAX	2

/* SA type and tuple info */
struct routeSA {
	u_long		transform;	/* use ./ipsec.h security types */
	struct routeSA_tuple {
		struct in_addr	src;
		struct in_addr	dst;
		u_long		spi;
	} tup[ROSA_MAX];
};
#endif /* IPSEC */

/* prototypes */

/* agent (or mn) add mn as link-layer route
*/
int routeInit();
void routeRead();
void route_add_link(struct in_addr mn, char *mac, char *ifname
#ifdef IPSEC
	, struct routeSA *rosa
#endif
	);
void route_delete_link(struct in_addr mn, char *mac);

void route_add_proxy(struct in_addr mn, char *mac, char *ifname);
void route_delete_proxy(struct in_addr mn);

void route_add_host(struct in_addr, struct in_addr 
#ifdef IPSEC
	, struct routeSA *
#endif
	);
void route_delete_host(struct in_addr, struct in_addr);

/* ha tunnel routes */
void ha_route_add_tunnel(struct in_addr mn, struct in_addr coa
#ifdef IPSEC
	, struct routeSA *rosa
        , struct routeSA *mvifrosa
#endif
	);
void ha_route_delete_tunnel(struct in_addr mn, struct in_addr coa);

/* function to allow linux to beacon even when it doesn't really have a route */
void mn_route_add_interface_default();
void mn_route_del_interface_default();

/* mn to agent as default router */
void mn_route_add_default(struct in_addr agent, struct in_addr interface,
	int dolink,  char *mac, char *ifname
#ifdef IPSEC
	, struct routeSA *rosa,
	u_long link_trans
#endif
	);
void mn_route_delete_default(struct in_addr router, int dolink);

/* new, only used with ad hoc routing
*/
void clone_route_add( struct in_addr cloneroute, struct in_addr interface, 
	struct in_addr subnet
#ifdef IPSEC
	, struct routeSA *rosa
#endif
	);

void reject_route_add( struct in_addr cloneroute, struct in_addr interface, 
	struct in_addr subnet);

void clone_route_delete(struct in_addr cloneaddr, struct in_addr mask);

void flushArp(void);

/* imported by route.h from *config.c */
int noDefaultRoute();

#ifdef IPSEC
int set_rosa(	int action,		/* TRUE add; FALSE delete */
		struct keylist *key,	/* SA source data, ipsec dst */
		struct in_addr src,	/* ipsec src */
		struct routeSA *rosap,	/* current rosa tuples */
		int identity,		/* type of agent/node holding rosa */
		struct in_addr who);	/* address of agent or node */
#endif
#endif

