/*
 * Copyright (c) 1996 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef IPSEC
/*
 *  routekey.h
 *
 *  DEPENDS upon mip_icmp.h
 */

#ifndef _MIP_ROUTEKEY_H
#define _MIP_ROUTEKEY_H

#define RKEYSIZE	128
#define MINSPI		256
#define RKEYBUCKETS	131
#define RKEYHASH(a) ((a) % RKEYBUCKETS)	/* input IP address */


struct routekey {
	struct sockaddr_in	src,	/* src/dst as given on input */
				dst;
	int			from,	/* SELF_MN, SELF_HA, SELF_FA,	*/
				to;	/*   or SELF_NONE (default!=0)	*/

	u_long			spi;
	int			type;		/* SECURITY_TYPE_* */
	int			transform;	/* IPSEC_ALGTYPE_* */
	int			keylen,
				ivlen;
	char			key[RKEYSIZE];
	char			iv[RKEYSIZE];

	int			used;	/* TRUE if key should be used */
	int			found;	/* TRUE if key in kernel */

	struct routekey		*next;
};



/* ========  prototypes  ======== */

int initKeySocket(void);

void rkey2kmsg(struct routekey *, int);
void kmsg2rkey(struct routekey *);

void keyMsg(int ksock, int msgtype);

void doRouteKey(char *op, char *linebuf, int linenum);
int verifyRouteKeys(void);

int addRouteKey(	struct routekey		**rkeylist,
			int			to,
			int			from,
                	struct sockaddr_in      *dst,
                	u_long                  spi,
			int		        type
			);

struct routekey * searchRouteKeyList(	struct routekey **rkeylist,
					struct in_addr src,
					struct in_addr dst,
					int type,
					u_long spi);

void dumpRouteKey(struct routekey *);
void dumpRouteKeyList(struct routekey **rkeylist);

#endif /* _MIP_ROUTEKEY_H */

#endif /* IPSEC */

