
#include "badicmp.h"


char *progname;

char packet[PACKETSIZEMAX];
u_long packetsize = 0;
u_long seq;

struct icmp		icmp, *icmp_p;
struct mip_mseicmp	mip, *mip_p;
int			s;	/* beacon socket */


void
main(int argc, char *argv[])
{
	struct sockaddr_in	iface;
	int			slen;
	
	if (getuid()) {
		printf("please be root.\n");
		exit(1);
	}

	/* parse commandline */
	progname = argv[0];
	parse(argc, argv);

	/* open socket, set defaults and construct a packet */
	initSocket();
	initPacket();

	bzero(&iface, slen = sizeof(iface));
	iface.sin_len = sizeof(iface);
	iface.sin_family = AF_INET;
	iface.sin_addr.s_addr = inet_addr(ROYALWLP0);	/* */
	/* iface.sin_addr.s_addr = inet_addr(BRIDGEPORTWL0);	/* */

	/* send forever */
	while (1) {
		/* set sequence, checksum */
		mip_p->mse_sequence = htons(seq++);
		icmp_p->icmp_cksum =
			in_cksum((u_short *)packet, (int)packetsize);

		/* output for the audience */
		showpacket("MIP ADVERTISEMENT");

		/* sendit */
		errno = 0;
		if (sendto(s, packet, packetsize, 0,
				(struct sockaddr *) &iface, slen) < 0)
		{
			perror("sendto");
		}
		
		sleep(BEACONPERIOD);
	}  /* endwhile */

	return;

}  /* end main() */


void
usage(int exitlevel, char *message) 
{
	if (message)
		printf("%s: %s\n", progname, message);

	printf(
  "Usage: %s ha|fa|hafa <#_coa> <#_routerAddresses> [<pref1> [<pref2>] ...]\n",
			progname);

	exit(exitlevel);

}  /* end usage() */


void
initSocket(void)
{
	struct protoent *proto;
	int on = 1;
	int rc;

	if ( !(proto = getprotobyname("icmp")) )
		usage(1, "unknown protocol icmp.");

	if ((s = socket(AF_INET, SOCK_RAW, proto->p_proto)) < 0) {
		usage(1, "socket (SOCK_RAW)");
	}

        if ((rc = setsockopt(s, SOL_SOCKET, SO_DONTROUTE,
						(char *)&on, sizeof(on))) < 0 )
	{
		usage(1, "setsockopt(SO_DONTROUTE)");
	}

        if ( (rc = setsockopt(s, SOL_SOCKET, SO_BROADCAST,
						(char *)&on, sizeof(on))) < 0 )
	{
		usage(1, "setsockopt(SO_BROADCAST)");
	}


        if ( (rc = setsockopt(s, SOL_SOCKET, SO_LIMITBCAST, (char *)&on,
                    sizeof(on))) < 0 )
	{
		usage(1, "setsockopt(SO_LIMITBCAST)");
	}

	/* build a packet given default values and user chosen values */

}  /* end initSocket() */


void
initPacket(void)
{
	caddr_t	p = (caddr_t) packet;
	u_long	i, ipaddr;

	icmp.icmp_type		= ICMP_ROUTERADVERT;
	icmp.icmp_code		= ICMPCODE;
	icmp.icmp_wpa		= ICMPADDRSIZE;	/* FIX -- test this too. */
	icmp.icmp_lifetime	= htons(ICMPLIFETIME);

	bcopy(&icmp, p, ICMP_MINLEN);
	icmp_p = (struct icmp *) p;

	p += ICMP_MINLEN;
	for (i = 0; i < icmp.icmp_num_addrs; i++, p += 8) {
		bcopy(&ralist[i].ira_addr, p, sizeof(u_int32_t));
		bcopy(&ralist[i].ira_preference, p+4, sizeof(u_int32_t));
	}

	mip.mse_type = MIPAGENTAD;
	mip.mse_lifetime = htons(MIPLIFETIME);

	bcopy(&mip, p, sizeof(mip));
	mip_p = (struct mip_mseicmp *) p;

	p += sizeof(mip);
	for (i = 0; coalist[i]; i++, p += 4) {
		bcopy(&coalist[i], p, sizeof(u_long));
	}

	mip_p->mse_len = p - (caddr_t) mip_p;
	packetsize = (u_long) (p - (caddr_t) packet);

}  /* end initPacket() */


void
parse(int argc, char *argv[])
{
	u_long numcoa = 0;
	u_long numrtaddr = 0;
	u_long i;
	

	if (argc < 3)
		usage(1, NULL);


	/* first argument */
	if (strcmp(argv[1], "ha") == 0) {
		mip.mse_flags = HABIT;
	} else if (strcmp(argv[1], "fa") == 0) {
		mip.mse_flags = FABIT;
	} else if (strcmp(argv[1], "hafa") == 0) {
		mip.mse_flags = HABIT|FABIT;
	} else {
		usage(1, NULL);
	}
	mip.mse_flags = htons(mip.mse_flags);

	/* second argument */
	if ((numcoa = (u_long) atol(argv[2])) < 0 && numcoa <= MAXCOA) {
		printf("%s: <#_coa> must be an integer no greater than %d.\n",
			progname, MAXCOA);
		usage(1, NULL);
	}

	for (i = 0; i < numcoa; i++)
		coalist[i] = htonl(ntohl(inet_addr(BASECOA)) + i);

	/* third argument */
	if ((numrtaddr = (u_long) atol(argv[3])) < 0 && numcoa <= MAXICMPRA) {
		printf("%s: <#_routerAddresses> must be an integer no greater than %d.\n",
			progname, MAXICMPRA);
		usage(1, NULL);
	}

	for (i = 0; i < numrtaddr; i++) {
		ralist[i].ira_addr =
			(u_int32_t) htonl(ntohl(inet_addr(BASEICMPRA))+i);
	}
	icmp.icmp_num_addrs = numrtaddr;

	/* remaning arguments */
	argc -= 4;
	argv += 4;
	i = 0;
	while (argc-- > 0) {
		ralist[i++].ira_preference = 
			(u_int32_t) htonl((u_long)atoi(*argv++));
	}

}  /* end parse() */


void
showpacket(char *header)
{
	u_long		i;
	caddr_t		p = (caddr_t)icmp_p + ICMP_MINLEN;
	char		*s = "<unknown>";
	u_int32_t	addr, pref;

	printf("\n  %s:\n", header);
	printf("\t(icmp)\ttype = %d\tcode = %d\tcksum = 0x%x\n",
		icmp_p->icmp_type, icmp_p->icmp_code, icmp_p->icmp_cksum);
	printf("\t\tnumaddr = %d\taddrlen = %d\tlifetime = %d\n\n",
		icmp_p->icmp_num_addrs, icmp_p->icmp_wpa,
		ntohs(icmp_p->icmp_lifetime));

	if (p == (caddr_t) mip_p) {
		printf("\t\t  <no Router Address(es)>\n");
	}
	else {
		for (i = 0; i < icmp_p->icmp_num_addrs; i++, p += 8) {
			bcopy(p, &addr, sizeof(u_int32_t));
			bcopy(p+4, &pref, sizeof(u_int32_t));
			printf("\t\t  %d\t%x  %d\n", i+1,
				(u_int32_t) ntohl(addr),
				(u_int32_t) ntohl(pref) );
		}
	}


	printf("\n\t(mip)\ttype = %d\tlength = %d\tsequence = %d\n",
		mip_p->mse_type, mip_p->mse_len, ntohs(mip_p->mse_sequence));


	switch(ntohs(mip_p->mse_flags)) {
	case HABIT|FABIT:
		s = "hafa";
		break;
	case HABIT:
		s = "ha";
		break;
	case FABIT:
		s = "fa";
		break;
	}

	printf("\t\tlifetime = %d\t\tagent is: %s\n\n",
		ntohs(mip_p->mse_lifetime), s);

	p += sizeof(mip);
	if (p == (caddr_t)(packet+packetsize)) {
		printf("\t\t  <no Care-Of Address(es)>\n");
	}
	else {
		for (i = 0; p != (caddr_t)(packet+packetsize); i++, p += 4)
		{
			printf("\t\t  %d\t%x\n", i+1, ntohl(*((u_long *) p)));
		}
	}

}  /* end showpacket() */


/*
 * in_cksum --
 *	Checksum routine for Internet Protocol family headers (C Version)
 *	(from BSD ping code)
 */
static u_short
in_cksum(u_short *addr, int len)
{
	register int nleft = len;
	register u_short *w = addr;
	register int sum = 0;
	u_short answer = 0;

	/*
	 * Our algorithm is simple, using a 32 bit accumulator (sum), we add
	 * sequential 16 bit words to it, and at the end, fold back all the
	 * carry bits from the top 16 bits into the lower 16 bits.
	 */
	while (nleft > 1)  {
		sum += *w++;
		nleft -= 2;
	}

	/* mop up an odd byte, if necessary */
	if (nleft == 1) {
		*(u_char *)(&answer) = *(u_char *)w ;
		sum += answer;
	}

	/* add back carry outs from top 16 bits to low 16 bits */
	sum = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
	sum += (sum >> 16);			/* add carry */
	answer = ~sum;				/* truncate to 16 bits */
	return(answer);

}  /* end in_cksum() */


/* FIX
	. multicast vs. broadcast beacons? (should not matter...)

	. need to test with adhoc authtication?
		
	. problems with 18 or more Router Addresses
	. problems with 17-19 COAs which affects flags bits

*/
