/* mnd.conf configuration routines */


#include <stdio.h>
#include <netinet/in.h>
#include <tcl.h>
#include <stdlib.h>

#include "mnconfig.h"
#include "unixsock.h"

#define MNHA	0
#define FAMN	1
#define NETAUTH	2
#define ADHOC	3

#define OFF 0
#define ON  1


unsigned long get_random();
void auth_policy(char);
void link_mode(char);
void change_key(int);

extern sock_des;

ET_PROC(mndconfigure)
{
	int i;
	char buf[4];
	char hold[10];
	char *policy_vals[]={"reject","accept"};
	char *bool[]={"off","on"};

	if (strcmp(argv[1], "read")==0){  /* read mnd.config file */
		if (configRead(CONFIGDEFAULT)<0){
			fprintf(stderr,"xmnstat: config read error\n");
			exit(1);
		}
		sprintf(hold, "%s|", policy_vals[mn_config.doNetauth]);
		strcat(interp->result, hold);
		sprintf(hold, "%s|", policy_vals[mn_config.doAdhoc]);
		strcat(interp->result, hold);
		strcat(interp->result, bool[mn_config.doArp]);
		return ET_OK;
	}
	else if (strcmp(argv[1],"write")==0){  /* write out mnd.conf */
		buf[0] = WRITECONF;
		if (sendUnixMessage(MNDSOCK_SERVER, buf, 1) < 0){
			perror("xmnstat: socket send message failed");
			close(sock_des);
			unlink(XMNDSOCK_CLIENT);
			exit(1);
		}
		return;
	}
	else if (strcmp(argv[1], "getkey")==0){ /* read current key */
		int j;
		char *keys[]={"mnha","famn","netauth","adhoc"};

		for (i=0; i<5; i++){
			if (strcmp(argv[2], keys[i])==0) break;
		}
		strcat(interp->result, "md5|");
		for (j=0;j<16;j++){
			sprintf(hold, "%2.2x", mn_config.sa[i].key[j]);
			strcat(interp->result, hold); 
		}
	}

	else {	/* its key entry time kids! */
		char *keys[]={"mnha","famn","netauth","adhoc"};
		for (i=0; i<3; i++){
			if (strcmp(argv[1],keys[i])==0) break;
		}
		if (pack_string(argv[2],mn_config.sa[i].key,32) < 0){
			fprintf(stderr, "%s, bad hex string\n",argv[2]);
			exit(1);
		}
		change_key(i);
	}
	return ET_OK; 
		
}

ET_PROC(xkeygen){
	unsigned int x1,x2,x3,x4;
	char hold[33];

	x1 = get_random();
	x2 = get_random();
	x3 = get_random();
	x4 = get_random();

	sprintf(hold, "%8.8x%8.8x%8.8x%8.8x",x1,x2,x3,x4);
	strcpy(interp->result, hold);
	return ET_OK;
}




ET_PROC(link_mode){
	char buf[2];

	buf[0] = LINKMODE;
	buf[1] = argv[1][0] - '0';

	if (sendUnixMessage(MNDSOCK_SERVER, buf, 2) < 0){
		perror("xmnstat: socket send message failed");
		close(sock_des);
		unlink(XMNDSOCK_CLIENT);
		exit(1); 
	}
	return;
}



/* change authentication policy */
ET_PROC(auth_policy)
{
	char buf[2];

	buf[0] = AUTHPOLICY;
	buf[1] = argv[1][0] - '0';

	if (sendUnixMessage(MNDSOCK_SERVER, buf, 2) < 0){
		perror("xmnstat: socket send message failed");
		close(sock_des);
		unlink(XMNDSOCK_CLIENT);
		exit(1);
	}
	return;
}

/* change keys in mnd */
void change_key(int type)
{
	char buf[1024];
	struct changekey temp;
	int i;

	buf[0] = AUTHCHANGEKEY;
	temp.change_type = type;
	memcpy(temp.key, mn_config.sa[type].key, 16);
	memcpy(buf+4, (char*)&temp, sizeof(struct changekey));

	if (sendUnixMessage(MNDSOCK_SERVER, buf, sizeof(struct changekey)+4) < 0){
		perror("xmnstat: socket send message failed");
		goto err;
	}
	return;

err:
	close(sock_des);
	unlink(XMNDSOCK_CLIENT);
	exit(1);
}
