/* xmnmain.c
* mostly embedded tcl/tk scripts in this file
*/
#include <tcl.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>


#ifdef IPSEC		/* needed to link with mnconfig.o */
int ipsecKeysInuse = 0;
#endif

char *progname;
int debug = 0;

void stopAlarm() {}
void startAlarm() {}

void main(int argc, char **argv)
{
	if (getuid()){
		fprintf(stderr, "xmnstat: root only.\n");
		exit(1);
	}
	progname = argv[0];
	Et_Init(&argc, argv);
	ET_INSTALL_COMMANDS(xmnprocs.c);
	ET_INSTALL_COMMANDS(xmnconfig.c);
	ET_INCLUDE(agents.tcl);
	ET_INCLUDE(mns.tcl);
	ET_INCLUDE(netauth.tcl);
	ET_INCLUDE(mndconf.tcl);
	ET_INCLUDE(statistics.tcl);
	ET_INCLUDE(state.tcl);
	ET_INCLUDE(help.tcl);
	ET_INCLUDE(beaconstats.tcl);
	ET_INCLUDE(utils.tcl);
	ET(
	# ----- global constants
	set STATE 0
	set AGENTS 1
	set STATS 2
	set STATE_DEBUG 3
	set MNS 4
	set BEAC_STATS 5
	set AUTH_CHANGEKEY 6
	set RESTART_MND 7

	set DO_STATS NO
	set DO_AGENTS NO
	set DO_STATE NO
	set DO_MNS NO
	set DO_BSTATS NO
	set DO_DUMBLIGHTS NO

	# auth policy change types
	set ACCEPT_NETAUTH 0
	set REJECT_NETAUTH 1
	set ACCEPT_ADHOC 2
	set REJECT_ADHOC 3

	# link mode change types
	set ARP_TO_AUTH 0
	set AUTH_TO_ARP 1

	# set max number of hosts to appear in histograms
	set MAX_MNS	10
	set MAX_AGENTS	10



	# ----- set variables for netauth function
	set arp 0
	set ask 0
	set net_auth 0
	set ad_hoc 0
	# dumblight variables
	set dl_val_beacs 0
	set dl_auth_badkey 0
	set dl_netauth_nokey 0
	set dl_arp_key 0

	# ----- set values for use in agents & mobile_ns functions
	set num_agents 0
	set num_mns 0
	set curr_agent NONE
	set curr_mn NONE
	set agent_colors [list green blue red yellow orange purple brown pink black] 
	set mn_colors [list orange purple brown pink green blue red yellow black] 
	foreach color $agent_colors {
        	set agent($color) "       "
		set agent_coa($color) 0
        	set sig_str($color) 0
		set ag_bar($color) 0
        }
	foreach color $mn_colors {
		set mn($color) "         "
		set mn_coa($color) 0
		set sig_str($color) 0
		set ag_bar($color) 0
	}
	# ----- set values for use in stats function
	set registrations 0
	set replies 0
	set beacons 0
	set udp_err_idm 0
	set udp_err_iha 0
	set udp_err_ifa 0
	set udp_err_hmtu 0
	set heartbeats 0
	set icmp_err_mts 0
	set icmp_err_ut 0
	set icmp_err_bip 0
	set icmp_err_iia 0
	set udp_err_mgr 0
	set udp_err_rts 0
	set udp_err_uxt 0

	# ----- set values for beacon stats function
	set ha_beacons_sent 0
	set fa_beacons_sent 0
	set mn_beacons_sent 0
	set all_beacons_recv 0
	set valid_beacons_recv 0
	set faisha_recv 0
	set tooshort 0
	set msetooshort 0
	set invalidicmpmse 0
	set bothhafa 0
	set auth_nomac 0
	set ha_netauthbad 0
	set fa_netauthbad 0
	set mn_netauthbad 0
	set ha_adhoc_authbad 0
	set fa_adhoc_authbad 0
	set mn_adhoc_authbad 0
	set ha_maybe_arp_lacks_auth 0
	set fa_maybe_arp_lacks_auth 0
	set mn_maybe_arp_lacks_auth 0
	set arp_on_mn_beacons_ignored 0
	set valid_arp_beacons 0
	set valid_auth_beacons 0


	# ---- set values for use in keys function
	#set key_type 0

# update function for main window  
proc get_state {} {
                global status_val agent_val timer_val STATE
                set col_code red
                global bar
                set status_values [split [get_mnd_info $STATE] |]
                set status_val [lindex $status_values 0]
                set timer_val [lindex $status_values 1]
                set temp [lindex $status_values 2]
                if {($temp != "notfound")||($status_val == "NOWHERE")} {
                        set agent_val $temp
                }
                set t_outs [lindex $status_values 3]
                if {$timer_val == "ACKED"} {
                        set t_outs 0
                }
                .timeouts delete $bar
                set bar [.timeouts create rectangle 0 .8c ${t_outs}c 1.2c -fill $col_code]
                after 1000 get_state
        }


	set bar 0
	#------------------------------------------
	#       set up main window layout
	#------------------------------------------

	wm title . "xmnstat - mnd user interface"
	frame .menu 
	frame .border1 -height 4 -relief ridge -borderwidth 4
	frame .border2 -height 4 -relief ridge -borderwidth 4
	frame .state_status -height 1c 
	frame .state_agent -height 1c
	frame .state_timer -height 1c
	pack .menu .border1 .state_status .state_agent .state_timer .border2 -side top -fill x

	frame .retry_status -height 1c -width 3c
	canvas .timeouts -height 2c -width 9c
	pack .retry_status .timeouts -side left

	#----- menu bars--------------------------
	menubutton .menu.ops -text "ops" -menu .menu.ops.options -underline 0
	menu .menu.ops.options
		.menu.ops.options add command -label "Status" -command state
		.menu.ops.options add command -label "stats" -command statistics
		.menu.ops.options add command -label "agents" -command agents
		.menu.ops.options add command -label "mns" -command mns
		.menu.ops.options add command -label "config" -command mndconf
	        .menu.ops.options add command -label "quit" -command {close_socket;exit}

	menubutton .menu.security -text "security" -menu .menu.security.options -underline 0
	menu .menu.security.options
	        .menu.security.options add command -label "netauth" -command netauth 
#	        .menu.security.options add command -label "Netsec" -command gecko
	menubutton .menu.help -text "help" -menu .menu.help.options -underline 0
	menu .menu.help.options
	        .menu.help.options add command -label "Status" -command {help status.hlp}
	        .menu.help.options add command -label "stats" -command {help stats.hlp}
	        .menu.help.options add command -label "agents" -command {help agents.hlp}

	pack .menu.ops .menu.security .menu.help -side left

	#------ state info --------------------------------
	label .state_status.des -text "STATUS: "
	label .state_status.val -textvariable status_val 
	pack .state_status.des .state_status.val -side left -padx 1m -pady 1m 
	label .state_agent.des -text "MOBILITY AGENT: "
	label .state_agent.val -textvariable agent_val 
	pack .state_agent.des .state_agent.val -side left -padx 1m -pady 1m 
	label .state_timer.des -text "REGISTRATION STATUS: "
	label .state_timer.val -textvariable timer_val
	pack .state_timer.des .state_timer.val -side left -padx 1m -pady 1m

	#------ retry info -------------------------------
	label .retry_status.des -text "RETRIES:"
	pack .retry_status.des 
	.timeouts create line 0 1.5c 9c 1.5c
	for {set d 1} {$d < 9} {incr d} {
	        .timeouts create line ${d}c 1.4c ${d}c 1.6c
	        .timeouts create text ${d}c 1.7c -text $d -anchor n
	}
	init_socket
	get_state
	);
	Et_MainLoop();
}

/*
logperror(char *s) 
{
        printf("%s\n", s);
}*/


