/* xmnprocs.c
*	c procedures for xmnstat
*/

#include <tcl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/time.h>
#include <string.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>

#include "unixsock.h"
#include "mnstats.h"
#include "mnstate.h"
#ifdef IPSEC
#include "keylist.h"
#include "route.h"
#endif
#include "alist.h"
#include "mnconfig.h"
#include "icmpstats.h"

/* global option constants */
#define STATE   '0'
#define AGENTS  '1'
#define STATS   '2'
#define STATE_DEBUG '3'
#define MNS	'4'
#define BEAC_STATS  '5'

#define LOC_LENGTH	12  /* length of location string */

/* booleans */
#define NO	0
#define YES	1

/* macros for statistics function */
#define ADDTMP          strcat(interp->result,tmp)
#define MKTMP(X)        sprintf(tmp, " %d",X)

/* window update functions, talk to mnd through socket connection */
void first_window(Tcl_Interp*);
void agents(Tcl_Interp*);
void mobile_ns(Tcl_Interp*);
void stats(Tcl_Interp*);
void beac_stats(Tcl_Interp*);
void state(Tcl_Interp*);
void dumpme(char *, int);
void logperror(char*);

struct mn_state mns;
int sock_des;

ET_PROC(init_socket)
{
	if ((sock_des=initUnixSocket(XMNDSOCK_CLIENT,FALSE))<0){
		perror("xmnstat: socket setup failed");
		goto err;
	}
	return ET_OK;
err:
	close(sock_des);
	unlink(XMNDSOCK_CLIENT);
	exit(1);
}

ET_PROC(close_socket)
{
	close(sock_des);
	unlink(XMNDSOCK_CLIENT);
	return ET_OK;
}



/* function creates interface between C and tcl code */
ET_PROC(get_mnd_info){

        if (argc < 2){
                fprintf(stderr,"xmnstat: invalid number of arguments.\n");
                exit(1);
        }
        switch(argv[1][0]){
                case STATE:
                        first_window(interp);
                        break;
                case AGENTS:
                        agents(interp);
                        break;
                case STATS:
                        stats(interp);
                        break;
		case STATE_DEBUG:
                        state(interp);
                        break;
		case MNS:
			mobile_ns(interp);
			break;
		case BEAC_STATS:
			beac_stats(interp);
			break;
                default:
                        fprintf(stderr, "error in get_mnd_info switch.\n");
                        exit(1);
        }
        return ET_OK;
}




/* primary application window */
void first_window(Tcl_Interp* interp)
{
	int acount, i;
	int found;
	char buf[1024];
	char t_outs[2];
	char *gen_status[] = {"NOWHERE", "ATHOME", "AWAY"};

	buf[0]=X_MNSTATE;
	if (sendUnixMessage(MNDSOCK_SERVER, buf, 1) < 0){
		perror("first_window():X_MNSTATE: socket send message failed");
		goto err;
	}
	if (recvUnixMessage((char*)&mns,sizeof(mns)) < 0){
		perror("xmnstat: socket recv message failed");
		goto err;
	}
	strcat(interp->result, gen_status[mns.state]);
	if (mns.retry_timer_flag)
		strcat(interp->result, "|PENDING|");
	else
		strcat(interp->result, "|ACKED|");

	/* cycle through agent list to get agent name */
	buf[0] = X_ALIST;
	found = NO;
	if (sendUnixMessage(MNDSOCK_SERVER, buf, 1) < 0){
		perror("first_window():X_ALIST: socket send message failed");
		goto err;
	}
	if (recvUnixMessage((char*)&acount,sizeof(acount)) < 0){
		perror("xmnstat: socket recv message failed");
		goto err;
	}
	for (i=0; i<acount; i++){
		struct alist a;
		if (recvUnixMessage((char *)&a,sizeof(a)) < 0){
			perror("xmnserv: socket recv message failed");
			goto err;
		}
		if ((mns.fa.s_addr == a.bip.s_addr)||((mns.state == ATHOME)&&(a.identity == OURHA))){   /* agent found */
			found = YES;
			if ( strcmp( a.location, "" )==0){ /* location null */
				char *tmp;
				tmp = inet_ntoa( a.bip );
				strncat( interp->result, tmp, LOC_LENGTH );
			}
			else{
				strcat(interp->result, a.location);
			}
		}
	}
	if (!found) /* current agent no longer in list */
		strcat(interp->result, "notfound");

	sprintf(t_outs, "|%d ", mns.retryIndex);
	strcat(interp->result, t_outs);
	return;

err:
	close(sock_des);
	unlink(XMNDSOCK_CLIENT);
}

/* agent signal strength window */
void agents(Tcl_Interp* interp)
{
	int acount;
	int current=-1;
	int i;
	char buf[1024];
	char hold[10];
	char *ptr;

	/* determine current agent */
	buf[0] = X_MNSTATE;
	if (sendUnixMessage(MNDSOCK_SERVER, buf,1)<0){
		perror("agents():X_MNSTATE: socket send message failed");
		goto err;
	}
	if (recvUnixMessage((char*)&mns,sizeof(mns))<0){
		perror("xmnstat: socket recv message failed");
		goto err;
	}
	
	/* read alist from mnd */
	buf[0] = X_ALIST;
	if (sendUnixMessage(MNDSOCK_SERVER, buf, 1) < 0){
		perror("agents():X_ALIST: socket send message failed");
		goto err;
	}
	if (recvUnixMessage((char*)&acount,sizeof(acount)) < 0){
		perror("xmnstat: socket recv message failed");
		goto err;
	}

	sprintf(hold, "%d|", acount);
	strcat(interp->result, hold);
	for (i=0; i<acount; i++){
		struct alist a;
		if (recvUnixMessage((char*)&a,sizeof(a)) < 0){
			perror("xmnstat: socket recv message failed");
			goto err;
		}
		if ((mns.fa.s_addr == a.bip.s_addr)||((mns.state == ATHOME)&&(a.identity == OURHA)))
			current = i;
		ptr = strchr(a.location,':');
		if (strcmp(a.location, "")==0){
			char *tmp;
			tmp = inet_ntoa( a.bip );
			strncat(interp->result, tmp, LOC_LENGTH );
		}
		else{ 
			/*char tmp[512]={};
			strncat( tmp, a.location, ((int)ptr)-((int)a.location ));
			*/
			strncat(interp->result, a.location, LOC_LENGTH ); 
		}
		if ( a.identity != OURHA ){	/*  a FA */
			sprintf(hold,"|%s",inet_ntoa(a.coa));
		}	
		else{ 		/* our ha, use bip instead of coa */
			sprintf( hold, "|%s", inet_ntoa( a.bip ));
		}
		strcat(interp->result, hold);
		sprintf(hold, "|%d|", a.sigaverage);
		strcat(interp->result, hold);
	}
	if (current> -1)
		sprintf(hold,"%d",current);
	else
		sprintf(hold, "NONE");
	strcat(interp->result,hold);
	return;
err:
	close(sock_des);
	unlink(XMNDSOCK_CLIENT);
	exit(1);
}

void mobile_ns(Tcl_Interp* interp)
{
	int acount;
	int i;
	char buf[1024];
	char hold[10];

	/* read mn list from mnd */
	buf[0] = X_MNLIST;
	if (sendUnixMessage(MNDSOCK_SERVER, buf, 1)<0){
		perror("mobile_ns():X_MNLIST: socket send message failed in mns");
		goto err;
	}
	if (recvUnixMessage((char*)&acount, sizeof(acount))<0){
		perror("xmnstat: socket recv message failed in mnd");
		goto err;
	}
	sprintf(hold, "%d|", acount);
	strcat(interp->result, hold);
	for (i=0; i<acount; i++){
		struct alist a;
		if (recvUnixMessage((char*)&a,sizeof(a)) < 0){
			perror("xmnstat: socket recv message failed");
			goto err;
		}
		/* if location is blank, use IP */
		if (strcmp(a.location, "")==0){
			strcat(interp->result, (char*)inet_ntoa(a.bip)); 
		}
		else {
			strncat(interp->result, a.location, LOC_LENGTH);
		}
		
		sprintf(hold,"|%s",inet_ntoa(a.coa));
		strcat(interp->result, hold);
		sprintf(hold, "|%d|", a.sigstrength);
		strcat(interp->result, hold);
	}
	strcat(interp->result,hold);
	return;
err:
	close(sock_des);
	unlink(XMNDSOCK_CLIENT);
	exit(1);
}



/* passes agent hint to mnd */
ET_PROC(hint){
        struct hintmsg hmsg;
        struct sockaddr_in agentsock;

        if (argc !=2) 
                return ET_OK;
	
	hmsg.code = U_MNHINT;
	if (resolveAddress(argv[1],&agentsock) == 0){
		fprintf(stderr, "agent address could not be resolved\n");
		return ET_OK;
	}

	memcpy((char*)hmsg.ip, (char*)&agentsock.sin_addr.s_addr, 4);

	if (sendUnixMessage(MNDSOCK_SERVER, (char*)&hmsg, sizeof(struct hintmsg))<0){
		perror("xmnstat: socket send msg failed");
		goto err;
	}
	return ET_OK;
err:
	close(sock_des);
	unlink(XMNDSOCK_CLIENT);
	exit(1);
}


/* statistics window */
void stats(Tcl_Interp* interp)
{
	char buf[1024];
	struct mn_stats mnstats;
	char tmp[10];

	buf[0] = X_MNSTAT;

	if (sendUnixMessage(MNDSOCK_SERVER, buf, 1) < 0){
		perror("stats():X_MNSTAT: socket send message failed.");
		goto err;
	}
	if (recvUnixMessage((char*)&mnstats,sizeof(mnstats)) < 0){
		perror("xmnstat: socket recv message failed.");
		goto err;
	}

	sprintf(tmp, "%d",mnstats.reg);
	ADDTMP;
	MKTMP(mnstats.replies);
	ADDTMP;
	MKTMP(mnstats.identnomatch);
	ADDTMP;
	MKTMP(mnstats.invalidhaauth);
	ADDTMP;
	MKTMP(mnstats.invalidfaauth);
	ADDTMP;
	MKTMP(mnstats.timeupdate);
	ADDTMP;
	MKTMP(mnstats.ticks);
	ADDTMP;
	MKTMP(mnstats.invalidregistration);
	ADDTMP;
	MKTMP(mnstats.repltooshort);
	ADDTMP;
	MKTMP(mnstats.udpbadtype);
	ADDTMP;
	return;
err:
	close(sock_des);
	unlink(XMNDSOCK_CLIENT);
	exit(1);
}

void beac_stats(Tcl_Interp* interp)
{
	char buf[1024];
	struct icmp_stats i_stats;
	char tmp[10];

	buf[0] = X_MNBEACONS;

	if (sendUnixMessage(MNDSOCK_SERVER, buf, 1)<0){
		perror("beac_stats():X_MNBEACONS: socket send message failed.");
		goto err;
	}
	if (recvUnixMessage((char*)&i_stats, sizeof(icmp_stats))<0){
		perror("xmnstat: socket recv message failed.");
		goto err;
	}

	sprintf(tmp, "%d", i_stats.ha_beacons_sent);
	ADDTMP;
	MKTMP(i_stats.fa_beacons_sent);
	ADDTMP;
	MKTMP(i_stats.mn_beacons_sent);
	ADDTMP;
	MKTMP(i_stats.all_beacons_recv);
	ADDTMP;
	MKTMP(i_stats.all_netauth_recv);
	ADDTMP;
	MKTMP(i_stats.valid_beacons_recv);
	ADDTMP;
	MKTMP(i_stats.faisha_recv);
	ADDTMP;
	MKTMP(i_stats.tooshort);
	ADDTMP;
	MKTMP(i_stats.msetooshort);
	ADDTMP;
	MKTMP(i_stats.invalidicmpmse);
	ADDTMP;
	MKTMP(i_stats.bothhafa);
	ADDTMP;
	MKTMP(i_stats.auth_nomac);
	ADDTMP;
	MKTMP(i_stats.ha_netauthbad);
	ADDTMP;
	MKTMP(i_stats.fa_netauthbad);
	ADDTMP;
	MKTMP(i_stats.mn_netauthbad);
	ADDTMP;
	MKTMP(i_stats.ha_adhoc_authbad);
	ADDTMP;
	MKTMP(i_stats.fa_adhoc_authbad);
	ADDTMP;
	MKTMP(i_stats.mn_adhoc_authbad);
	ADDTMP;
	MKTMP(i_stats.ha_maybe_arp_lacks_auth);
	ADDTMP;
	MKTMP(i_stats.fa_maybe_arp_lacks_auth);
	ADDTMP;
	MKTMP(i_stats.mn_maybe_arp_lacks_auth);
	ADDTMP;
	MKTMP(i_stats.arp_on_mn_beacons_ignored);
	ADDTMP;
	MKTMP(i_stats.valid_arp_beacons);
	ADDTMP;
	MKTMP(i_stats.valid_auth_beacons);
	ADDTMP;
	return;
err:
	close(sock_des);
	unlink(XMNDSOCK_CLIENT);
	exit(1);
}




/* function to get mnd's pid */
ET_PROC(restart_mnd)
{
	char buf[1];
	int pid;
	int rc; 

	buf[0] = GETMNDPID;

	if (sendUnixMessage(MNDSOCK_SERVER, buf, 1) < 0){
		perror("ET_PROC(restart_mnd):GETMNDPID: socket send message failed");
		goto err;
	}
	if (recvUnixMessage((char*)&pid, sizeof(int)) < 0){
		perror("xmnstat: socket recv message failed");
		goto err;
	}

	/* send signal to restart mnd */
	rc = kill(pid, SIGUSR1);
	if (rc < 0){
		perror("xmnstat: kill error");
		goto err;
	}
	/* reopen xmnstat socket */
	close( sock_des );
	unlink(XMNDSOCK_CLIENT);

	sleep(5); 	/* give mnd time to recover */

	if (( sock_des = initUnixSocket( XMNDSOCK_CLIENT, FALSE ))<0){
		perror("ET_PROC(restart_mnd): socket reset error" );
		goto err;
	}

	return ET_OK;
err:
	close(sock_des);
	unlink(XMNDSOCK_CLIENT);
	exit(1);
}
		


		
	
		
	


	

void state(Tcl_Interp* interp)
{
	int sd;
	char buf[1024];
	char tmp[20];

	buf[0]=X_MNSTATE;
	if (sendUnixMessage(MNDSOCK_SERVER, buf, 1) < 0){
		perror("state():X_MNSTATE: socket send message failed");
		goto err;
	}
	if (recvUnixMessage((char*)&mns,sizeof(mns)) < 0){
		perror("xmnstat: socket recv message failed");
		goto err;
	}
	switch(mns.state){
		case NOWHERE:
			strcat(interp->result, "NOWHERE|");
			break;
		case ATHOME:
			if (mns.retry_timer_flag) {
				strcat(interp->result, "ATHOME-PENDING|");
			}
			else {
				strcat(interp->result, "ATHOME|");
			}
			break;
		case AWAY:
			if (mns.retry_timer_flag)
               		         if (mns.retryTimeouts)
                       		        strcat(interp->result,"AWAY-PENDING-DISCONNECTED?|");
                        	else
                                	strcat(interp->result, "AWAY-PENDING|");
                	else
                        	strcat(interp->result,"AWAY-ACKED|");
               		break;
	}

        if (mns.fa.s_addr != 0) {
		sprintf(tmp, "%s|", inet_ntoa(mns.fa));
   		strcat(interp->result, tmp);
		sprintf(tmp, "%s|", inet_ntoa(mns.coa));
        	strcat(interp->result, tmp);
        }
        else
                strcat(interp->result, "mns.fa/coa 0|");

        sprintf(tmp, "%d|", mns.haroute);
	ADDTMP;
	sprintf(tmp, "%x %x|",mns.ident[0],mns.ident[1]);
	ADDTMP;
	sprintf(tmp, "%ld|", mns.seqno);
	ADDTMP;
	sprintf(tmp, "%d %d|", mns.lifetime_flag, mns.lifetime);
	ADDTMP;
	sprintf(tmp, "%d %d|", mns.reg_timer_flag, mns.reg_timer);
	ADDTMP;
	sprintf(tmp, "%d %d|", mns.retry_timer_flag, mns.retry_timer);
	ADDTMP;
	sprintf(tmp, "%d|",mns.retryIndex);
	ADDTMP;
	sprintf(tmp, "%d|", mns.retryTimeouts);
	ADDTMP;
	sprintf(tmp, "%d %d|", mns.hint_flag, mns.hint_set);
	ADDTMP;
	sprintf(tmp, "%d", mns.docoa);
	ADDTMP;
	return;

err:
	close(sd);
	unlink(XMNDSOCK_CLIENT);
	exit(1);
}
	

void logperror(char *s) 
{
        printf("%s\n", s); 
}


/* file utility functions */
ET_PROC(rm){
	unlink(argv[1]);
	return ET_OK;
}

ET_PROC(cp){
	link(argv[1], argv[2]);
	return ET_OK;
}
	


